﻿using System;
using UnityEngine;
using UnityEngine.UI;

namespace JudgeModule
{
    public class NoteJudger
    {
        private Transform deactives;
        private Action<int> EndGame;
        private GameObject judgeText;

        public NoteJudger(Action<int> endgame)
        {
            deactives = GameObject.Find("Deactives").    transform;
            judgeText = GameObject.Find("Judge");
            EndGame = endgame;
        }

        public void WrongNoteProc(Note note)
        {
            SetJudge(Judge.MISS);
            DeactivateNote(note);

            if (!(note is MotionNote))
                EndGame(0);
        }

        public void EnteredNoteProc(Note note, float timing)
        {
            var judge = Judge.TestJudge(note, timing);

            SetJudge(judge);

            if (note.IsLong && !judge.IsBreak)
                note.Activated = true;
            else
                DeactivateNote(note);
        }

        public void ContinuingNoteProc(Note note, float timing)
        {
            var isEnd = Judge.IsNoteEnd(note, timing);
            var judge = Judge.TestJudge(note, timing, isEnd);

            SetJudge(judge);

            if (!isEnd && 
                judge.IsBreak)
                DeactivateNote(note);
        }

        public void CorrectlyStoppedNoteProc(Note note, float timing)
        {
            SetJudge(Judge.TestJudge(note, timing, true));
            DeactivateNote(note);
        }

        public void IncorrectlyStoppedNoteProc(Note note, float timing = 0)
        {
            SetJudge(Judge.MISS);
            DeactivateNote(note);
        }

        public void DeactivateNote(Note note)
        {
            note.Component.Deactivate();
            note.Component.transform.SetParent(deactives);
        }

        public void SetJudge(Judge judge)
        {
            AllSceneManager.Combo = judge.IsBreak ? 0 : AllSceneManager.Combo + 1;
            AllSceneManager.Score += judge.Score;
            AllSceneManager.JudgeCount[judge]++;

            judgeText.SetActive(true);
            judgeText.GetComponent<Text>().text = judge.Name;
            judgeText.GetComponent<Text>().color = judge.Color;
        }
    }
}